#  Badmunda - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Badmunda.
#
#  Badmunda is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Badmunda is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Badmunda.  If not, see <http://www.gnu.org/licenses/>.

from Badmunda import raw

from ..object import Object


class SavedCredentials(Object):
    """Contains information about saved payment credentials.

    Parameters:
        id (``str``):
            Unique identifier of the saved credentials.

        title (``str``):
            Title of the saved credentials.
    """
    def __init__(
        self,
        *,
        id: str,
        title: str
    ):
        super().__init__()

        self.id = id
        self.title = title

    @staticmethod
    def _parse(credential: "raw.base.PaymentSavedCredentials") -> "SavedCredentials":
        return SavedCredentials(
            id=credential.id,
            title=credential.title
        )
